#!/bin/bash

#PROCS=1    # number of processors used in parallel
TIMEOUT=300 # wall clock timeout for single ConCon call
ALL=(1.5 1.7)

PATH=$PATH:~/ttt2
PATH=$PATH:~/csi-1.2.2-x86_64
PATH=$PATH:~/wm-feb18-linux/bin

function ConCon() {
  CONCON=""
  case $2 in
    "1.5")
      CONCON="./concon-1.5 -s"
      ;;
    *)
      CONCON="./concon-1.7 -s"
      ;;
  esac
  timeout `expr $TIMEOUT + 10` $CONCON $1 $TIMEOUT
}

function doit() {
for f in cops/*.trs; do
  cops=$(basename ${f%%.trs}) # cops number
  tmpfile=$(mktemp)
  echo -n "$f:"
  ConCon $f $1 &> "$tmpfile"
  answer=$(cat "$tmpfile" | head -n 1)
  echo "${answer}"
  if [[ ${answer} == "YES" || ${answer} == "NO" ]]
  then
    cat $tmpfile | tail -n +2 > proofs/$1/${cops}.txt
  fi
  rm $tmpfile
done
}

export -f doit ConCon
export CONCON TIMEOUT PROCS

function run() {
  mkdir -p "proofs/$1"
  (time doit "$1" | tee "$1.RESULTS") 2> "$1.time"
}

function stats() {
  for x in $@
  do
    printf "%3s - " $x
    f=$x.RESULTS
    if [[ -f $f ]]
    then
      Y=$(grep -c YES $f)
      N=$(grep -c NO $f)
      K=$(grep -c '\(YES\|NO\)' $f)
      M=$(grep -c MAYBE $f)
      T=$(grep -c TIMEOUT $f)
      U=$(grep -c '\(MAYBE\|TIMEOUT\)' $f)
      printf "YES: %3s  NO: %3s (KNOWN: %3s)  " $Y $N $K
      printf "MAYBE: %3s  TIMEOUT: %3s (UNKNOWN: %3s)  " $M $T $U
      echo
    else
      echo "no results available"
    fi
  done
}

function usage()
{
  cat <<EOF

Usage: ${PRG} [OPTIONS] {run [1.5 | 1.7] | stats [1.5 | 1.7] | compare}

  Run experiments.
  Available options:

    -j n            number of parallel processes (default ${PROCS})
    -t n            timeout (in seconds; default ${TIMEOUT})

EOF
  exit $1
}

while getopts ":j:t:" OPT
do
  case "${OPT}" in
    j)
      PROCS=${OPTARG}
      ;;
    t)
      TIMEOUT=${OPTARG}
      ;;
    \?)
      echo "Invalid option: -${OPTARG}" >&2
      usage 1 >&2
      ;;
  esac
done

shift $((${OPTIND} - 1))

if [[ $1 == "stats" ]]
then
  shift
  if [[ $# -ne 0 ]]
  then
    stats $@
  else
    stats ${ALL[@]}
  fi
elif [[ $1 == "compare" ]]
then
  echo "$(diff -y 1.5.RESULTS 1.7.RESULTS | grep '|' | wc -l) differences"
  printf "%10s  %60s\n" 1.5 1.7
  diff -y 1.5.RESULTS 1.7.RESULTS | grep '|'
elif [[ $1 == "run" ]]
then
  shift
  if [[ $# -ne 0 ]]
  then
    for x in $@; do echo $x; eval "run $x"; done
  else
    for x in ${ALL[@]}; do echo $x; eval "run $x"; done
  fi
else
  echo "command '$1' not known" >&2
  usage 1 >&2
fi

